/* fishguts.h - Fishnet internals*/

#ifndef __FISHGUTS_H__
#define __FISHGUTS_H__

/* headers to handle structures listed here */
#include <stdio.h>
#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#include <sys/timeb.h>
#define bzero(a,len) memset(a,0,len)
#include <io.h>
#define STDIN_FILENO 0
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>
#endif
#include <sys/types.h>

#ifdef __LCLINT__
/*@-incondefs@*/
const char /*@dependent@*/*inet_ntoa(struct in_addr);
int accept(int, /*@out@*/struct sockaddr *, socklen_t *);
/*@+incondefs@*/
#endif

#include "fish.h"

#define DEFAULT_PORT 7777
#define DEFAULT_HOST 0
#define MAX_NEIGHBORS 12

struct neighbor {
  int address;                    /* chosen address, HOST order */
  struct in_addr node_ip_address; /* ip address NETWORK order */
  unsigned short node_udp_port;   /* udp address NETWORK order */
  unsigned short lossrate_x_1000; /* loss rate across this link */
};

struct joinrequest {
  int node_address;               /* chosen address, HOST order */
  unsigned short node_udp_port;   /* udp address NETWORK order */
  short reserved;                 /* 32-bit alignment padding */
  char username[8];               /* for ownership reporting */
  char domain[8];                 /* for hierarchical topologies */
};

#define FISH_MSG_TYPE_ERROR        0
#define FISH_MSG_TYPE_NEIGHBORLIST 1
#define FISH_MSG_TYPE_JOINREQUEST  2
#define FISH_MSG_TYPE_MAX          3 /* upper bound */

struct fish_message {
  unsigned short length;    /* HOST order (may change later) */
  unsigned short type;
  union {
    /* zero size arrays terminating a message can be
       referenced with a longer array index */
    char errormessage[0]; /* null terminated, 0 included in
                             length */
    struct neighbor neighbors[0]; /* terminated by message
                                     length */
    struct joinrequest joinreq; 
  } body;
  /* put nothing here  (neighbors will be out-of-bounds) */
};

#define MIN_MESSAGE_SIZE 4

/* common routines */
/* the following is gcc specific */
#ifndef WIN32
#define DEBUG_PRINTF(fmt,args...) fprintf(stderr, __FILE__ ":%d" fmt, \
                                                  __LINE__ , ## args )
#else
#define DEBUG_PRINTF(x) do {} while(0)
#endif

/* in fishtimer.c */
void fish_executepending(void);
int fish_time_to_next_event(struct timeval *delta_t);

/* in fishdebug.c */
extern void printmessage(const struct fish_message *pfm);
extern void printneighbors(const struct neighbor *ngh);

void errorabort(const char* msg);
#endif
